searchData={"items":[{"type":"task","title":"mix tamnoon.make_dirs","doc":"Creates a \"components\" directory inside \"lib\" and a \"tamnoon_out\" directory in the root of the project.","ref":"Mix.Tasks.Tamnoon.MakeDirs.html"},{"type":"module","title":"Tamnoon","doc":"This module provides functions needed to initialize Tamnoon. You do not need to handle\nit directly, rather, the only time you need to call something in this module is in your\nsupervision tree, to add it to the children and configure it (see `child_spec/1`).","ref":"Tamnoon.html"},{"type":"module","title":"Example - Tamnoon","doc":"```\ndef start_link(opts \\\\ []) do\n  children = [Tamnoon]\n  opts = [strategy: :one_for_one, name: Tamnoon.Supervisor]\n  Supervisor.start_link(children, opts)\nend\n```","ref":"Tamnoon.html#module-example"},{"type":"function","title":"Tamnoon.child_spec/1","doc":"Returns a Tamnoon server supervisor child spec. See `t:tamnoon_opts/0` for more info.","ref":"Tamnoon.html#child_spec/1"},{"type":"function","title":"Tamnoon.make_release/1","doc":"Copies HEEx file components to the release directory and creates a _tamnoon_out_ directory\nin it. It is needed to be ran as a _step_ in the release (see `m:Mix.Release`).","ref":"Tamnoon.html#make_release/1"},{"type":"function","title":"Tamnoon.start_link/1","doc":"Starts the supervisor. See `t:tamnoon_opts/0` for more info.","ref":"Tamnoon.html#start_link/1"},{"type":"type","title":"Tamnoon.tamnoon_opts/0","doc":"Options for initializing Tamnoon. Defaults to `[8000, Tamnoon.Router, Tamnoon.SocketHandler, Tamnoon.Methods, %{}]`.\n- `port`: The port Tamnoon will run on. Defaults to _8000_.\n- `initial_state`: The state new clients will start with. Defaults to an empty map.\n- `methods_module`: The module where your methods are defined (see `m:Tamnoon.Methods`). Defaults to `m:Tamnoon.Methods`.\n- `router`: The router module (see `m:Plug.Router`). Defaults to `m:Tamnoon.Router`.\n- `socket_handler`: The handler module for WebSocket requests. Usually doesn't need to be overriden. Defaults to `m:Tamnoon.SocketHandler`.\n- `protocol_opts`: Whether Tamnoon uses HTTP or HTTPS. See `t:tamnoon_protocol_opts/0` for more info.","ref":"Tamnoon.html#t:tamnoon_opts/0"},{"type":"type","title":"Tamnoon.tamnoon_protocol_opts/0","doc":"Options for configuring the protocol used by Tamnoon. Can be either `:http` or a keyword list containing\nvalues for `:keyfile`, `:certfile`, and `:otp_app` *(`:otp_app` is required only when using relative paths for the\nkey and certificate files)*. Defaults to `:http`.","ref":"Tamnoon.html#t:tamnoon_protocol_opts/0"},{"type":"module","title":"Tamnoon.Compiler","doc":"This module provides functions for compiling (/parsing) Tamnoon HEEx components.","ref":"Tamnoon.Compiler.html"},{"type":"function","title":"Tamnoon.Compiler.build_from_root/1","doc":"Compiles the 'root.ex' component into the final file that is sent to the client.","ref":"Tamnoon.Compiler.html#build_from_root/1"},{"type":"function","title":"Tamnoon.Compiler.escape_html/1","doc":"Escapes the content so it is not rendered as HTML. Available as `<%= h.(content) %>`\r\ninside components.","ref":"Tamnoon.Compiler.html#escape_html/1"},{"type":"function","title":"Tamnoon.Compiler.parse_tmnn_heex/1","doc":"Takes as an argument a string containing compiled heex for a component, and returns the\r\ncomponent as plain HTML for it to be rendered on the DOM.","ref":"Tamnoon.Compiler.html#parse_tmnn_heex/1"},{"type":"function","title":"Tamnoon.Compiler.render_component/3","doc":"Returns the compiled HTML code of the component. Takes as arguments:\r\n- either a component module or a file name of a heex component file inside _lib/components_,\r\n- the _assigns_ for the EEx blocks in the component,\r\n- and a boolean for whether to parse the Tamnoon heex.\r\nWhen the function is used inside a component that gets built into the root in the `build_from_root/2`\r\nfunction, the last argument should be `false` to avoid invoking `parse_tmnn_heex/1` multiple\r\ntimes.\r\nInside EEx blocks in heex components, this function can be invoked as `<%= r.([args..]) %>`.\r\nSee `render_component_dyn/1`.","ref":"Tamnoon.Compiler.html#render_component/3"},{"type":"function","title":"Tamnoon.Compiler.render_component_dyn/1","doc":"Utility function passed to EEx blocks in components. Calls `render_component/3` with the\r\nelements in the list as the arguments. When passing it only the first argument, it can be\r\npassed directly without putting it in a list.\r\nSee `render_component/3` for more info.","ref":"Tamnoon.Compiler.html#render_component_dyn/1"},{"type":"behaviour","title":"Tamnoon.Component","doc":"This module defines the `m:Tamnoon.Component` behaviour, which includes one callback -\r\n`c:heex/0`. It is good practice to implement this behaviour in a component module, however\r\nas long as you define a function named _heex_ taking 0 arguments and returning the heex,\r\nit should work just fine.","ref":"Tamnoon.Component.html"},{"type":"callback","title":"Tamnoon.Component.heex/0","doc":"","ref":"Tamnoon.Component.html#c:heex/0"},{"type":"module","title":"Tamnoon.Components.Root","doc":"","ref":"Tamnoon.Components.Root.html"},{"type":"module","title":"Tamnoon.MethodManager","doc":"This module handles the management of different methods as you create them.\r\nNotably, it provides the `defmethod/2` macro.\r\n> #### Importing the module {: .info}\r\n> In order to create handlers for the methods you set up, you must `import Tamnoon.MethodManager`\r\n> in your _methods module_. Then, you can use the `defmethod/2` macro to implement handling\r\n> of the methods.","ref":"Tamnoon.MethodManager.html"},{"type":"macro","title":"Tamnoon.MethodManager.defmethod/2","doc":"Defines a function named `tmnn_[name]`. Functions with this prefix in your _methods module_\r\nwill automatically be added to the possible methods when invoking `route_request/3`.\r\nInside the created function, you can access the state of the client and the request\r\nobject by the `state` and `req` variables respectively.\r\nMethod handlers must return a tuple of `{return_value, new_state}`.","ref":"Tamnoon.MethodManager.html#defmethod/2"},{"type":"macro","title":"Example - Tamnoon.MethodManager.defmethod/2","doc":"```\r\ndefmethod :get do\r\n  key = get_key(req, state)\r\n  if (key != nil) do\r\n    {state[key], state}\r\n  else\r\n    {\"Error: no matching key\", state}\r\n  end\r\nend\r\n```","ref":"Tamnoon.MethodManager.html#defmethod/2-example"},{"type":"function","title":"Tamnoon.MethodManager.route_request/3","doc":"The function used by `Tamnoon.SocketHandler.websocket_handle/2` to route the requests\r\nto the appropriate method handler.","ref":"Tamnoon.MethodManager.html#route_request/3"},{"type":"module","title":"Tamnoon.Methods","doc":"Provides default implementations for methods you need to get your Tamnoon server\r\nworking - basic state management (with `tmnn_get/2` and `tmnn_update/2`) and basic handling of\r\nPubSub channels (via `tmnn_sub/2`, `tmnn_unsub/2`, `tmnn_pub/2`, `tmnn_subbed_channels/2`)\r\n> #### Using the module {: .info}\r\n> When you `use Tamnoon.Methods`, every method in this module will be imported for you,\r\n> letting you conveniently make a _methods module_. See `m:Tamnoon.MethodManager` for\r\n> details on making one.\r\n\r\n_Note: in the documentation of the default methods, the term 'returns' is used both\r\nto describe the return value of the function and the value of the server's response._","ref":"Tamnoon.Methods.html"},{"type":"function","title":"Tamnoon.Methods.get_key/2","doc":"Returns the value of the key under the field `\"key\"` in the request as an atom.\r\nNote: this is *NOT* a method handler - simply a utility function.","ref":"Tamnoon.Methods.html#get_key/2"},{"type":"function","title":"Tamnoon.Methods.tmnn_get/2","doc":"Returns the field with the key specified under the `\"key\"` field\r\nin the request. Returns an error string if there is no such item.","ref":"Tamnoon.Methods.html#tmnn_get/2"},{"type":"function","title":"Tamnoon.Methods.tmnn_keep_alive/2","doc":"Invoked every 55 seconds by the client in order to prevent idle timeouts.","ref":"Tamnoon.Methods.html#tmnn_keep_alive/2"},{"type":"function","title":"Tamnoon.Methods.tmnn_pub/2","doc":"Will send the request under the `\"action\"` key in the request to every client\r\nin the channel under the `\"channel\"` key.","ref":"Tamnoon.Methods.html#tmnn_pub/2"},{"type":"function","title":"Tamnoon.Methods.tmnn_set_state/2","doc":"Sets the state on the server to the state sent by the client. Invoked when the client reconnects\r\nto the server in order to prevent information loss.","ref":"Tamnoon.Methods.html#tmnn_set_state/2"},{"type":"function","title":"Tamnoon.Methods.tmnn_sub/2","doc":"Subscribes to the channel under the `\"key\"` field in the request. If there is no such\r\nchannel, one will be created (with the client subscribed to it).\r\nIf the client is already subscribed to the channel nothing will happen.","ref":"Tamnoon.Methods.html#tmnn_sub/2"},{"type":"function","title":"Tamnoon.Methods.tmnn_subbed_channels/2","doc":"Returns a list of the channels the client is currently subscribed to.","ref":"Tamnoon.Methods.html#tmnn_subbed_channels/2"},{"type":"function","title":"Tamnoon.Methods.tmnn_sync/2","doc":"Returns the current state. Automatically invoked when the connection is started.","ref":"Tamnoon.Methods.html#tmnn_sync/2"},{"type":"function","title":"Tamnoon.Methods.tmnn_unsub/2","doc":"Unsubscribes from the channel under the `\"key\"` in the request.","ref":"Tamnoon.Methods.html#tmnn_unsub/2"},{"type":"function","title":"Tamnoon.Methods.tmnn_update/2","doc":"Updates the value of the item with the key at the `\"key\"` field on the request,\r\nsetting it to the value at the `\"val\"` key. Returns a tuple with the updated field and\r\nthe state, or an error string if there is no such item.","ref":"Tamnoon.Methods.html#tmnn_update/2"},{"type":"module","title":"Tamnoon.Router","doc":"This module provides a default router for HTTP(S) requests, which builds and serves the web app.","ref":"Tamnoon.Router.html"},{"type":"function","title":"Tamnoon.Router.call/2","doc":"","ref":"Tamnoon.Router.html#call/2"},{"type":"function","title":"Tamnoon.Router.init/1","doc":"","ref":"Tamnoon.Router.html#init/1"},{"type":"module","title":"Tamnoon.SocketHandler","doc":"A default `:cowboy_websocket` implementation. Usually there should not be a reason to do anything\r\ndirectly with this module, as it gets its options from `Tamnoon.start_link/1` and\r\nadapts to them automatically. However, it is still documented for the sake of allowing\r\nextensiblity.\r\nYou can replace the module with a custom one via setting it as the _(drumroll..)_ `:socket_handler`\r\nkey's value in `Tamnoon.start_link/1`'s options.","ref":"Tamnoon.SocketHandler.html"},{"type":"function","title":"Tamnoon.SocketHandler.init/2","doc":"Initiates the websocket connection.","ref":"Tamnoon.SocketHandler.html#init/2"},{"type":"function","title":"Tamnoon.SocketHandler.websocket_handle/2","doc":"Receives a websocket request and decodes it, sending it to the `Tamnoon.MethodManager.route_request/3`\r\nfunction.","ref":"Tamnoon.SocketHandler.html#websocket_handle/2"},{"type":"function","title":"Tamnoon.SocketHandler.websocket_info/2","doc":"Handles messages from other BEAM processes. By default it is only used for handling\r\n`\"pub\"` requests, and as such it simply forwards the message to `websocket_handle/2`.","ref":"Tamnoon.SocketHandler.html#websocket_info/2"},{"type":"function","title":"Tamnoon.SocketHandler.websocket_init/1","doc":"Initiates the websocket and adds the client to the registry under the `\"clients\"` channel.","ref":"Tamnoon.SocketHandler.html#websocket_init/1"},{"type":"extras","title":"Tamnoon","doc":"# Tamnoon\n\n#### A simplicity-first web framework for Elixir.\n\nTamnoon is a web framework aiming to provide a simple, minimal and approachable solution to web development with Elixir. It takes a component-based and functional approach, which makes Tamnoon apps easily debuggable.\n\nIn addition to easy client-server communication, it comes with built-in PubSub functionality too - allowing you to sync up multiple users effortlessly.\n\nTo get started, check out the _Overview_ guide.","ref":"readme.html"},{"type":"extras","title":"Installation - Tamnoon","doc":"The package can be installed by adding `tamnoon` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:tamnoon, \"~> 1.0.0-a.3\"}\n  ]\nend\n```\n\nThe docs can be found at  .","ref":"readme.html#installation"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"V1.0.0-a.3 (25.07.24) - Changelog","doc":"- Added support for running Tamnoon over HTTPS.\r\n\r\n- Added support for Mix Releases (via `Tamnoon.make_release/1`). \r\n\r\n- Updated the client script to try reconnecting to the server if it disconnects.\r\n\r\n- Updated the client script to keep a copy of the state which will be sent to the server on a reconnect. _Note: this also means that string keys in the state are now not supported, as the state will have its keys converted to atoms on the reconnect._\r\n\r\n#### Minor Changes:\r\n\r\n- Added `Tamnoon.Methods.tmnn_set_state/2`.\r\n\r\n- Added deployment guide (WIP)","ref":"changelog.html#v1-0-0-a-3-25-07-24"},{"type":"extras","title":"v1.0.0-a.2 (29.06.24) - Changelog","doc":"- Added **implicit events**: setting an input event attribute (such as onchange) to `@method-key` will make it fire the method with the specified key. The `\"pub\"` method can be triggered like so: `@pub-[channel]-[method]-[key (optional)]`.\r\n\r\n- A component with a Tamnoon variable inside it (` @val `) will now set the inner text of the element instead of the inner HTML. The inner HTML can be set by prefixing the variable name with \"raw-\" like so: ` @raw-val `.\r\n\r\n- Added the `mix tamnoon.make_dirs` task that creates a \"lib/components\" directory and a \"tamnoon_out\" directory.\r\n\r\n- Added `Tamnoon.Compiler.escape_html/1` and made it available as `<%= h.(content) %>` inside components.\r\n\r\n- Completely rewrote the guides, the readme, and revamped some of the existing module documentation.\r\n\r\n#### Minor changes:\r\n\r\n- Changed the default port to 8000 from 4000.\r\n\r\n- Fixed \"imported function conflicts with local function\" when invoking a method handler inside the methods module.\r\n\r\n- Fixed the client-side handling of updates for elements with the \"value\" attribute not functioning properly.\r\n\r\n- Changed the WebSocket script in the root layout to connect to the WebSocket at the current URL (instead of _localhost:8000/ws_).\r\n\r\n- Changed `Tamnoon.Compiler.build_from_root/1` to no longer accept a WebSocket address.","ref":"changelog.html#v1-0-0-a-2-29-06-24"},{"type":"extras","title":"v1.0.0-a.1 (26.06.24) - Changelog","doc":"- Added the _sync_ method (`Tamnoon.Methods.tmnn_sync/2`) that returns the current state to the client. By default, it is invoked when the WebSocket connection is opened.\r\n\r\n- Added the _keep\\_alive_ method (`Tamnoon.Methods.tmnn_keep_alive/2`) that is invoked every 55 seconds by the client in order to prevent idle timeouts.\r\n\r\n- Moved the WebSocket connection script to a separate file, and updated the default router (`m:Tamnoon.Router`) to serve a default root layout.\r\n\r\n#### Minor changes:\r\n\r\n- Changed the console message sent when Tamnoon starts to include the full address.\r\n\r\n- Changed `Tamnoon.SocketHandler.init/2` so connections have a 120 second idle timeout (instead of 60).\r\n\r\n- Added `Tamnoon.Compiler.render_component_dyn/1`.\r\n\r\n- Changed `Tamnoon.Compiler.render_component/3` so using `<%= r.(args..) %>` will call `Tamnoon.Compiler.render_component_dyn/1` instead of `Tamnoon.Compiler.render_component/1`, to allow passing in multiple arguments.\r\n\r\n- Added (temporary) documentation.","ref":"changelog.html#v1-0-0-a-1-26-06-24"},{"type":"extras","title":"v1.0.0-a.0 (25.06.24) - Changelog","doc":"Added support for HEEx components and changed the default router to display the root page by default.","ref":"changelog.html#v1-0-0-a-0-25-06-24"},{"type":"extras","title":"v0.1.1 - Changelog","doc":"Very minor changes.","ref":"changelog.html#v0-1-1"},{"type":"extras","title":"v0.1.0 - Changelog","doc":"First release.","ref":"changelog.html#v0-1-0"},{"type":"extras","title":"Overview","doc":"# Overview\r\n\r\nTamnoon is a web development framework for Elixir, designed with a \"simplicity-first\" approach. It aims to deliever a smooth and efficient development experience, and to be as approachable as possible. \r\n\r\nOut of the box, Tamnoon comes with built-in state management and PubSub functionality. These guides will familiarize with Tamnoon, while also providing a step-by-step example of developing a small, real-time chat room app.\r\n\r\n_Note: If you only wish to see an example of Tamnoon's workflow, check out the Wrapping Up guide._","ref":"overview.html"},{"type":"extras","title":"Guides Summary - Overview","doc":"Below is a rundown of the contents of every guide:\r\n\r\n- _Getting Started_: Installation and basic setup.\r\n\r\n- _Components_: Building the UI. By this point you should be able to create static web pages with Tamnoon.\r\n\r\n- _The State and Methods_: Introduction to real-time client-server interactions.\r\n\r\n- _Tamnoon HEEx_: The Tamnoon HEEx syntax, and how to use it in order to implement real-time client-server interactions.\r\n\r\n- _Wrapping Up_: Completing the chat app.\r\n\r\n-_Deployment_: Deploying Tamnoon apps (Mix Releases, HTTPS).","ref":"overview.html#guides-summary"},{"type":"extras","title":"Getting Started","doc":"# Getting Started\r\n\r\nTo get started with Tamnoon, make a new app with a supervision tree:\r\n\r\n```console\r\n$ mix new [NAME] --sup\r\n```\r\n\r\nThen, add Tamnoon to your `deps` in `mix.exs`:\r\n\r\n```\r\n  defp deps do\r\n    [\r\n      # Other dependencies...\r\n      {:tamnoon, \"~> 1.0.0-a.2\"}\r\n    ]\r\n  end\r\n```\r\n\r\nWe can now get started! Simply add Tamnoon to your supervision tree as such:\r\n\r\n```\r\n  def start(_type, _args) do\r\n    children = [\r\n      {Tamnoon}\r\n    ]\r\n    opts = [strategy: :one_for_one, name: TamnoonTest.Supervisor]\r\n    Supervisor.start_link(children, opts)\r\n  end\r\n```\r\n\r\nYou should now be able to run `mix run --no-halt` and get the following message in your console:\r\n\r\n```console\r\n10:34:54.305 [info] Tamnoon listening on http://localhost:8000..\r\n```\r\n\r\nIf everything went correctly, you should a blank page when visiting http://localhost:8000 on your browser. The next guide will go over the basic workflow with Tamnoon.","ref":"getting-started.html"},{"type":"extras","title":"Components","doc":"# Components\r\n\r\nIn this guide, we will start writing the UI. From this point onwards, we will build a chat room app and add things to it in every guide.\r\n\r\nIn Tamnoon, the UI is made up of _components_. Components are reusable elements written in Tamnoon HEEx - an extension of HEEx (HTML + Elixir). Meaning, every HEEx code is valid Tamnoon HEEx code. Components can either be elixir modules that implement the `m:Tamnoon.Component` behaviour, or _.html.heex_ files located inside the `lib/components` directory. There is no difference between the two besides modules being able to utilize all of Elixir's features.\r\n\r\n_Note: We will get back to Tamnoon HEEx later. For now, think of it as normal HEEx._\r\n\r\nWhen a user retrieves the page, the \"root\" layout gets parsed and is returned. The root layout is a component module just like any other, and this is (partially) how it looks:\r\n\r\n```\r\ndefmodule Tamnoon.Components.Root do\r\n  @behaviour Tamnoon.Component\r\n\r\n  @impl true\r\n  def heex do\r\n    ~s\"\"\"\r\n    <!DOCTYPE html>\r\n     \r\n      ...\r\n       \r\n     \r\n\r\n     \r\n         \r\n            <%= r.(\"app.html.heex\") %>\r\n         \r\n     \r\n     \r\n    \"\"\"\r\n  end\r\nend\r\n```\r\n\r\nAs you can see, it simply contains one function named \"heex\" that returns the Tamnoon HEEx. Note the `<%= r.(\"app.html.heex\") %>`: This is an EEx block, that renders a component (In this case `\"app.html.heex\"`). Inside components you can use the `r.()` syntax to invoke `Tamnoon.Compiler.render_component_dyn/1`, which is the function that renders a component and can pass assigns to it.\r\n\r\nLet's make the \"app\" component. Run `mix tamnoon.make_dirs`, which will generate a components directory and an output directory. Now, make a file named \"app.html.heex\" inside of your components directory (located at \"lib/components\"). Let's test that it works by writing the following inside of it:\r\n\r\n```\r\n  1 + 1 = <%= 1 + 1 %>  \r\n```\r\n\r\nRun `mix run --no-halt`, visit _http://localhost:8000_, and you should see a blank page with the text _1 + 1 = 2_. Our Elixir block got parsed and our app is working as intended!","ref":"components.html"},{"type":"extras","title":"Making More Components - Components","doc":"Let's make another component that will read the time from an _assign_, and will display a chat message. This time, let's make it a module. Inside your components directory, make a file named `message_box.ex` and write the following in it:\r\n\r\n```\r\ndefmodule TamnoonChatroom.Components.MessageBox do\r\n  @behaviour Tamnoon.Component\r\n  \r\n  @impl true\r\n  def heex do\r\n    ~s\"\"\"\r\n     \r\n        <%= h.(@user_name) %>  \r\n        <%= h.(@content) %>  \r\n     \r\n    \"\"\"\r\n  end\r\nend\r\n```\r\n\r\nIn the component, we are rendering the `@user_name` and `@content` assigns. We are also using the `h.()` function, which is a shortcut for `Tamnoon.Compiler.escape_html/1` similar to `r.()`. This will prevent HTML injections to the page by escaping the necessary characters.\r\n\r\nNow let's use it inside `\"app.html.heex\"`. Add the following to the file, restart the server and reload the page.\r\n\r\n```\r\n<%= r.([TamnoonChatroom.Components.MessageBox, %{user_name: \"user1\", content: \"  this is my message  \"}]) %>\r\n```\r\n\r\nYou should now see \"user1\" displayed in bold text and the content of the message displayed under it, including the ` ` tags.\r\n\r\nAt this point, you have enough knowledge to create static pages. In the next guide we will go over the state and methods, which are how Tamnoon provides real-time updates.","ref":"components.html#making-more-components"},{"type":"extras","title":"The State and Methods","doc":"# The State and Methods","ref":"the-state-and-methods.html"},{"type":"extras","title":"The State - The State and Methods","doc":"In Tamnoon, every app keeps a _state_. The state is a map consisting of multiple _variables_, which can be read and changed by the app. It can be thought of similarly to an agent.\r\nTo set the initial state, change Tamnoon's `child_spec` in your supervision tree as such:\r\n\r\n```\r\n{Tamnoon, [[initial_state: %{msgs: [], name: \"\", curr_msg: \"\"}]}\r\n```\r\n\r\nAs you can see, we are storing the user's name, the message the user is currently typing, and the messages.\r\n\r\n_Note: The state should not have string keys, only atoms._","ref":"the-state-and-methods.html#the-state"},{"type":"extras","title":"Methods - The State and Methods","doc":"When we want something to happen, we need to trigger _methods_. Methods are the client's way of interacting with the server. Tamnoon comes with a few different built-in methods for interacting with the state and for implementing PubSub functionality (more on that later):\r\n\r\n- `get`: Returns the value of the requested variable to the client.\r\n\r\n- `update`: Sets the value of the requested variable to the the value specified in the request, and returns the new value to the client.\r\n\r\n- `sync`: Returns all of the state to the client.\r\n\r\n- `sub` and `unsub`: Subscribe and unsubscribe the client from the requested channel. \r\n\r\n- `pub`: Sends the request under the \"action\" field to all the other clients in the requested channel.\r\n\r\n- `subbed_channels`: Returns a list of the channels the client is currently subscribed to.\r\n\r\n**Tamnoon abstracts this whole process, so you don't need to worry about it.** However, FYI: methods are triggered by the client sending a json object to the server. The `update` and `get` methods both expect a field with the key `\"key\"`, which is a name of a variable in the state. `update` also expects a field with the key `\"val\"` which is the new value of the variable.","ref":"the-state-and-methods.html#methods"},{"type":"extras","title":"Custom Methods - The State and Methods","doc":"You may also create additional methods if you wish. To do so, you need to make a _methods module_. Make a file named \"methods.ex\" inside \"lib\", and add the following content to it:\r\n\r\n```\r\ndefmodule TamnoonChatroom.Methods do\r\n  use Tamnoon.Methods\r\n\r\n  defmethod :log do\r\n    IO.inspect(req)\r\n    IO.inspect(state)\r\n    {nil, state}\r\n  end\r\nend\r\n```\r\n\r\nHere we are utilizing the `Tamnoon.MethodManager.defmethod/2` macro, imported by the `use Tamnoon.Methods` line. A method can access the `state` and `req` variables, and needs to return a tuple containing the value to return to the client (usually a map) and the updated state.\r\n\r\nBefore we can test it, we need to set the module as our _methods module_. Inside your _application.ex_, change the `start` function as such:\r\n\r\n```\r\ndef start(_type, _args) do\r\n    children = [\r\n      {Tamnoon, [[initial_state: %{msgs: [], name: \"\", curr_msg: \"\"},\r\n      methods_module: TamnoonChatroom.Methods]]}\r\n    ]\r\n    # ...\r\n  end\r\nend\r\n```\r\n\r\nTo test it, let's create a button to trigger the method. add the following to your `\"app.html.heex\"` file:\r\n\r\n```\r\n push me! \r\n```\r\n\r\nFor now, don't worry about the _onclick=@log_ part. That is Tamnoon HEEx, which we will go over in the next guide. Once you restart the server and reload the page, you should see the following output on your console:\r\n\r\n```\r\n%{\r\n  \"element\" => \" push me! \",\r\n  \"method\" => \"log\",\r\n  \"val\" => \"\"\r\n}\r\n%{name: \"\", msgs: [], curr_msg: \"\"}\r\n```\r\n\r\nWe can now move on to Tamnoon HEEx, which will allow you to interact with the server in real-time.","ref":"the-state-and-methods.html#custom-methods"},{"type":"extras","title":"Tamnoon HEEx","doc":"# Tamnoon HEEx\r\n\r\nTamnoon HEEx is an extension of HEEx (HTML + Elixir), which makes it easy to add real-time functionality.","ref":"tamnoon-heex.html"},{"type":"extras","title":"Reading Variables - Tamnoon HEEx","doc":"When we want to read a variable from the state in our app (in this example we will display the user's name), we can simply include it as `@variable` in a component as such:\r\n\r\n```\r\n  can you see me?  \r\n @name \r\n @raw-name \r\n```\r\n\r\n- The first _ _ will be hidden if we have a method that returns `%{hidep: true}` to the client. We don't need to have a variable named _hidep_ in our state for that work, however most often you will want to.\r\n\r\n- The second _ _ will have its text updated every time the _name_ variable changes. \r\n\r\n- The last _ _ will have its inner _HTML_ updated every time _name_ changes. This is a way to dynamically switch between components.","ref":"tamnoon-heex.html#reading-variables"},{"type":"extras","title":"Updating Variables (Events) - Tamnoon HEEx","doc":"Now let's update our variables. Consider the following:\r\n\r\n```\r\n push me! \r\n \r\n```\r\n\r\n- As seen previously, the button will trigger the `log` method when pressed.\r\n\r\n- The input will trigger the `update` method for the key _name_. \r\n\r\nThese are _events_. By default, events send the server an object with fields for the method, the outer HTML of the element, the value of it, and if a _-_ is used then also a key. _(There is additional syntax for using `pub`, we will go over it in the PubSub guide)_","ref":"tamnoon-heex.html#updating-variables-events"},{"type":"extras","title":"Example - Tamnoon HEEx","doc":"Let's continue our chat room. Inside `\"app.html.heex\"` add the following:\r\n\r\n```\r\n Name:  @name   \r\n \r\n```\r\n\r\nRestart the server, reload the page, and you should now see the name change as you type!","ref":"tamnoon-heex.html#example"},{"type":"extras","title":"Wrapping Up","doc":"# Wrapping Up\r\n\r\nIn this \"guide\", we will utilize the different features to create a chat app, continuing from the previous guides. The source code for the result is available on [github](https://github.com/omer-sm/tamnoon_chatroom).\r\n\r\nFirst of all, let's delete everything in our `\"app.html.heex\"` and start from fresh. Now, add inputs for the user's name and the message:\r\n\r\n```\r\n Name:   @name    \r\n \r\n \r\n \r\n > \r\n```\r\n\r\nThe topmost input will change the user's name every time it changes, and those changes will be reflected in the h4 above it. Currently the other input and the button don't really do anything, so let's change that by adding a _send\\_msg_ method in our _methods module_:\r\n\r\n```\r\n  defmethod :send_msg do\r\n    {msg, new_state} = Map.get_and_update(state, :curr_msg, &({&1, \"\"}))\r\n    msg = Tamnoon.Compiler.render_component(TamnoonChatroom.Components.MessageBox, %{\r\n      user_name: state[:name],\r\n      content: msg\r\n    }, true)\r\n    {%{pub: :ok}, new_state} = pub(%{\r\n      \"channel\" => \"clients\",\r\n      \"action\" =>\r\n      %{\"method\" => \"add_msg\",\r\n        \"msg\" => msg}\r\n    }, new_state)\r\n    {%{\"curr_msg\" => \"\"}, new_state}\r\n  end\r\n```\r\n\r\nThis will clear the value of the _\"curr\\_msg\"_ input, and _pub_ a request of the _add\\_msg_ method. Let's define that method as well:\r\n\r\n```\r\n  defmethod :add_msg do\r\n    msg = req[\"msg\"]\r\n    {_old_msgs, new_state} = Map.get_and_update(state, :msgs, &({&1, [msg | Enum.take(&1, 4)]}))\r\n    msgs_ret = Enum.reduce(new_state[:msgs], fn x, acc ->\r\n      x <> acc\r\n    end)\r\n    {%{\"msgs\" => msgs_ret}, new_state}\r\n  end\r\n```\r\n\r\nThis will add the new message to _\"msgs\"_, make sure there are 5 messages at most, and return all the messages as snippet of HTML code to the client. To display our messages, we simply add a _ _ with `@raw-msgs` as its inner value to `\"app.html.heex\"`. This will display the content of _\"msgs\"_ as HTML. Now, `\"app.html.heex\"` looks like this:\r\n\r\n```\r\n Name:   @name    \r\n \r\n \r\n  @raw-msgs  \r\n \r\n \r\n > \r\n```\r\n\r\nAll that is left is to restart the server, reload the page, and we now have a functioning chat!\r\n\r\n![Final Chat App Preview](assets/chat.gif)","ref":"wrapping-up.html"},{"type":"extras","title":"Deployment","doc":"# Deployment\r\n\r\nThis guide will go over the process of deploying Tamnoon apps in various ways.","ref":"deployment.html"},{"type":"extras","title":"HTTPS - Deployment","doc":"To deploy your app, you will most likely need to configure it to run over HTTPS. Fortunately, this is simple. In your supervision tree, add the following options to Tamnoon:\r\n\r\n```\r\n  def start(_type, _args) do\r\n    children = [\r\n      {Tamnoon, [[other options..., protocol_opts: [keyfile: PATH TO YOUR KEYFILE, certfile: PATH TO YOUR CERTFILE, otp_app: YOUR OTP APP]]]}\r\n    ]\r\n    ...\r\n  end\r\n```\r\n\r\n_Note: the `:otp_app` is required only if you use relative paths for the keyfile / certfile._\r\n\r\nRun the app using `mix run --no-halt` as usual, and your app should now be served over HTTPS.","ref":"deployment.html#https"},{"type":"extras","title":"Using Releases - Deployment","doc":"To deploy your Tamnoon app on a server, you will most likely want to package it into a _release_. A release is a self contained package including everything needed to run your app. For more information see [`Mix.Release`](https://hexdocs.pm/mix/Mix.Tasks.Release.html).\r\n\r\nIn order to assemble your release, you will need to go into your `mix.exs` file and add the following inside `def project` (replace NAME with your desired name for the release):\r\n\r\n```\r\n  def project do\r\n    [\r\n      ...\r\n      releases: [\r\n        NAME: [\r\n          steps: [:assemble, &Tamnoon.make_release/1]\r\n        ]\r\n      ]\r\n    ]\r\n  end\r\n```\r\n\r\nOnce that is done, run `mix release NAME` with the name you provided in the previous step. Mix will assemble your release, and eventually you should get an output similar to the following:\r\n\r\n```console\r\nRelease created at _build/dev/rel/NAME\r\n\r\n    # To start your system\r\n    _build/dev/rel/NAME/bin/NAME start\r\n\r\nOnce the release is running:\r\n...\r\n```\r\n\r\nNow, once you go to the specified directory and run the `start` command, the server should start as normal. You can now package that directory and transfer it to any machine you would like to!","ref":"deployment.html#using-releases"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,50,46,50]}}